package com.xqual.xlauncher;

import java.io.File;
import java.util.Vector;

import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CMailFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.utils.CUtils;

public class CClientDownloader implements IConstantsResults {

	// +==============================================================+
	// | Members                                                      |
	// +==============================================================+

	static final String TRACE_HEADER = "    {client dwnld  }  ";

	private final CLauncher launcher;
	private File localInstallerFile;

	private final String smtpHost;
	private final String smtpPort;
	private final String smtpFrom;
	private final String smtpTo;
	private final String username;
	private final String password;
	private boolean sslEnabled = false;
	private final boolean tlsEnabled = false;
	private final Vector<String> ccRecipients;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CClientDownloader(CLauncher launcher) {
		this(launcher, null, null, null, null, null, null, false, null);
	}

	public CClientDownloader(CLauncher launcher,
							 String smtpHost,
							 String smtpPort,
							 String smtpFrom,
							 String smtpTo,
							 String username,
							 String password,
							 boolean sslEnabled,
							 Vector<String> ccRecipients) {
		this.launcher = launcher;
		this.smtpHost = smtpHost;
		this.smtpPort = smtpPort;
		this.smtpFrom = smtpFrom;
		this.smtpTo = smtpTo;
		this.username = username;
		this.password = password;
		this.sslEnabled = sslEnabled;
		this.ccRecipients = ccRecipients;
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	public short downloadClient(String downloadServerUrl,
	                            String installerTemplateName,
								String installerMajorVersion,
								String sutVersion,
								String localInstallersFolder
								) {
		// prepare the mailer
    	CMailFactory mailFactory = smtpHost == null ? null : new CMailFactory(null, smtpHost, smtpPort, smtpFrom,
    	                                                                        username, password,
    	                                                                        sslEnabled, tlsEnabled, true);

		short status = RESULT_FAILURE;

		if (sutVersion.equals("...latest...")) {
			System.out.println("looking for the latest version...");
	    	CSharedFolderDownloader downloader = new CSharedFolderDownloader();

	        // get the list of the available releases
	        Vector<Integer> releaseNumberVector = downloader.getReleaseNumbers(downloadServerUrl);
	        Vector<Integer> sortedVector = CUtils.sortIntegerVector(releaseNumberVector);

	        if (releaseNumberVector.size() > 0) {
		        int index = sortedVector.indexOf(sortedVector.lastElement());
		        int nbFailure = 0;
		        StringBuffer failedBuilds = new StringBuffer();
		        Integer latestValidRelease = 0;
		        while (status == RESULT_FAILURE) {
		        	if (index < 0) {
		        		System.out.println("status: failure (no more candidate)");
		        		status = RESULT_FAILURE;
		        		break;
		        	}

		        	latestValidRelease = sortedVector.get(index);

		        	String installerToTestFileName = installerTemplateName.replaceFirst("%version%", installerMajorVersion + "." + latestValidRelease);
		        	localInstallerFile = new File(localInstallersFolder + "/" + installerToTestFileName);

					System.out.println("check if " + localInstallerFile + " already exist locally...");
		        	if (localInstallerFile.exists()) {
		        		System.out.println(localInstallerFile + " already exist locally");
		        		status = RESULT_SUCCESS;
		        		break;
		        	} else {
		        		System.out.println(localInstallerFile + " does not already exist locally");
		        	}

		        	System.out.println("installer " + localInstallerFile + " does not exist on the file system");
		        	System.out.println("trying to download release " + latestValidRelease + "...");

					boolean downloadSucceeded = downloader.downloadFile(localInstallerFile,
					                                                    downloadServerUrl + "/" + installerToTestFileName,
					                                                    true);
		            if (downloadSucceeded) {
		            	System.out.println("status: success");

		        		// update the sutVersion
		        		launcher.setSutVersion(installerMajorVersion + "." + latestValidRelease);
		        		launcher.setSutMustBeCreated(true);

		            	status = RESULT_SUCCESS;
		            } else {
		            	System.out.println("status: failure");
		            	nbFailure++;
		            	failedBuilds.append(" " + latestValidRelease);
		            	index--;
		            	status = RESULT_FAILURE;
		            }
		        } // while

		        if (mailFactory != null) {
			        if (nbFailure>1) {
			        	mailFactory.sendMail(smtpTo,
			        						 ccRecipients,
			        			             "[Build Issue]",
			        			             "This email has been automatically generated by the test framework because<br>" +
			        			             nbFailure + " consecutive builds have failed recently:<br>" +
			        			             failedBuilds);
			        } else {
			        	mailFactory.sendMail(smtpTo,
								 	         "[Build Issue]",
								 	         "This email has been automatically generated by the test framework.<br>" +
								 	         "The build " + latestValidRelease + " has been successfully automatically downloaded");
			        }
		        }
	        } else { // if (releaseNumberVector.size() > 0)... else ...
	        	System.out.println("status: no release were found");
	        	status = RESULT_FAILURE;
	        }

		} else { // if (sutVersion.equals("...latest...")) ... else ...

			System.out.println("looking for a specific version...");

			// determine the buildVersion
			int lastDotIndex = sutVersion.lastIndexOf(".");              // i.e. "0.9.4.1234"
			String majorVersion = sutVersion.substring(0, lastDotIndex); // i.e. "0.9.4"
			String buildVersion = sutVersion.substring(lastDotIndex+1);  // i.e. "1234"

			// check if the installer is already present
			String targetInstallerFileName = installerTemplateName.replaceFirst("%version%", sutVersion); // i.e. "sut-setup-0.9.4.1234.exe"
			localInstallerFile = new File(localInstallersFolder + "/" + targetInstallerFileName);

			if (!localInstallerFile.exists()) {
				System.out.println("installer " + localInstallerFile + " does not exist on the file system");

				CSharedFolderDownloader downloader = new CSharedFolderDownloader();
	        	boolean downloadSucceeded = downloader.downloadFile(localInstallerFile,
	        	                                                    downloadServerUrl + "/" + targetInstallerFileName,
	        	                                                    true);
	            if (downloadSucceeded) {
	            	System.out.println("status: download succeeded");
	        		status = RESULT_SUCCESS;

			        if (mailFactory != null) {
			        	mailFactory.sendMail(smtpTo,
								 	         "[Build Issue]",
								 	         "This email has been automatically generated by the test framework.<br>" +
								 	         "The build " + buildVersion + " has been successfully automatically downloaded");
			        }

	            } else {
	            	System.out.println("status: download failed");
	            	status = RESULT_FAILURE;

			        if (mailFactory != null) {
			        	mailFactory.sendMail(smtpTo,
					   						 ccRecipients,
					   			             "[Build Issue]",
					   			             "This email has been automatically generated by the test framework.<br>" +
					   			             "the download of the build " + buildVersion + " failed for any reason");
			        }
	            }
			} else {
				status = RESULT_SUCCESS;
			}
		}
		return status;
	}

	public short installSilentlyClient() {
		CRunner install = new CRunner("install silently the client", localInstallerFile.toString() + " /S");
		short result = install.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
		if (result != RESULT_SUCCESS) {
			System.out.println("silent install failed!");
			return RESULT_FAILURE;
		} else {
			return RESULT_SUCCESS;
		}
	}
}
