package com.xqual.xlauncher;

import java.io.File;
import java.util.Vector;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xcommon.CFileWriter;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;

public class CLauncherCommonParsers implements IConstantsResults {

	// +==============================================================+
	// | Members                                                      |
	// +==============================================================+

	private static final String TRACE_HEADER = "    {launch. parser}  ";

	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	static {
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	public static CReturnStatus parseJUnitXmlFile(File inputFile, File extractedOutputFile, String testcaseName, Vector<CExecutionStep> executionSteps) {
		System.out.println("+----------------------------------------------------------+");
		System.out.println("| Standard Parser: JUnit XML                               |");
		System.out.println("+----------------------------------------------------------+");
		Document inputDocument = CXmlDocumentFactory.createXMLDoc(inputFile);
		
		// parse the result file to get the result and the execution steps
		if (!inputFile.exists() || inputDocument == null) {
			System.err.println("Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
		}

		// search in the file for the right testcase
		Node testcaseNode = CXmlDocumentFactory.getNodeFromXPath(inputDocument, "//testcase[@name='" + testcaseName + "']");
		if (testcaseNode == null) {
			System.err.println("Testcase not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: testcase \"" + testcaseName + "\" not found!"));
			return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps);
		}
		
		System.out.println("Testcase \"" + testcaseName + "\" found");
		
		if (extractedOutputFile != null) { // if we need an extracted version
			CFileUtils.deleteAndTouchFile(extractedOutputFile);

			// store the part of the XML file that corresponds to this testcase as an attachment
			String partedXmlOutput = CXmlDocumentFactory.nodeToString(testcaseNode);
			CFileWriter.write(extractedOutputFile, partedXmlOutput);
		}
		
		// parse the log
		String _className = null;
		if (testcaseNode.getAttributes().getNamedItem("classname") != null)
			_className = testcaseNode.getAttributes().getNamedItem("classname").getNodeValue();
		String _testcaseName    = testcaseNode.getAttributes().getNamedItem("name").getNodeValue();
		String _testcaseTime    = testcaseNode.getAttributes().getNamedItem("time").getNodeValue();
		System.out.println(">> testcase: class=" + _className + " name=" + _testcaseName + " time=" + _testcaseTime);
		
		
		if (_className != null)
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]class/name[/b]: " + _className + "/" + _testcaseName));
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]time[/b]: " + _testcaseTime));

		System.out.println("Searching for testcase's system out...");
		String _outCDataLog     = "";
		Node testcaseOutLogNode = CXmlDocumentFactory.getNodeFromXPath(inputDocument, "//testcase[@name='" + testcaseName + "']/system-out");
		if (testcaseOutLogNode != null) {
			_outCDataLog = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseOutLogNode);
			System.out.println(">>           system-out=" + _outCDataLog);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]system-out[/b]: " + _outCDataLog));
		}
		
		

		System.out.println("Searching for testcase's failure...");
		String _failureType     = "";
		String _failureMessage  = "";
		String _failureCDataLog = "";
		Node testcaseFailureLogNode = CXmlDocumentFactory.getNodeFromXPath(inputDocument, "//testcase[@name='" + testcaseName + "']/failure"); // present only when there is a failure
		if (testcaseFailureLogNode != null) {
			_failureType     = null;
			if (testcaseFailureLogNode.getAttributes().getNamedItem("type") != null)
			try {_failureMessage  = testcaseFailureLogNode.getAttributes().getNamedItem("message").getNodeValue();} catch (Exception e) {}
			try {_failureType     = testcaseFailureLogNode.getAttributes().getNamedItem("type").getNodeValue();} catch (Exception e) {}
			try {_failureCDataLog = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseFailureLogNode);} catch (Exception e) {}
			System.out.println(">>           failure type=" + _failureType + " message=" + _failureMessage);
			System.out.println(">>           failure=" + _failureCDataLog);
			
			if (_failureType != null)
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "[b]type[/b]: " + _failureType));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]message[/b]: " + _failureMessage));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]failure[/b]: " + _failureCDataLog));
		}

		System.out.println("Searching for testcase's error...");
		String _errorType     = "";
		String _errorMessage  = "";
		String _errorCDataLog = "";
		Node testcaseErrorLogNode = CXmlDocumentFactory.getNodeFromXPath(inputDocument, "//testcase[@name='" + testcaseName + "']/error"); // present only when there is an error
		if (testcaseErrorLogNode != null) {
			_errorType     = testcaseErrorLogNode.getAttributes().getNamedItem("type").getNodeValue();
			_errorMessage  = testcaseErrorLogNode.getAttributes().getNamedItem("message").getNodeValue();
			_errorCDataLog = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseErrorLogNode);
			System.out.println(">>           error type=" + _errorType + " message=" + _errorMessage);
			System.out.println(">>           error=" + _errorCDataLog);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "[b]type[/b]: " + _errorType));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]message[/b]: " + _errorMessage));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]error[/b]: " + _errorCDataLog));
		}

		
		System.out.println("Searching for testcase's system err...");
		String _errCDataLog     = "";
		Node testcaseErrLogNode = CXmlDocumentFactory.getNodeFromXPath(inputDocument, "//testcase[@name='" + testcaseName + "']/system-err");
		if (testcaseErrLogNode != null) {
			_errCDataLog = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseErrLogNode);
			System.out.println(">>           system-err=" + _errCDataLog);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]system-err[/b]: " + _errCDataLog));
		}

		
		return new CReturnStatus((testcaseFailureLogNode != null || testcaseErrorLogNode != null) ? RESULT_FAILURE : RESULT_SUCCESS, executionSteps);
	}
	
}
