package com.xqual.xlauncher;

import java.io.File;

import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.utils.CUtils;

public class CTimeoutListener implements IConstantsResults {

	// +==============================================================+
	// | Members                                                      |
	// +==============================================================+

	static final String TRACE_HEADER = "    {timeout listen}  ";

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CTimeoutListener() {
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	/*
	 * timeout in seconds
	 */
	public static short waitForFile(File file) {
		return waitForFileWithData(file, -1, 0, 0); 
	}
	
	public static short waitForFile(File file, int timeout) { // check timeout
		return waitForFileWithData(file, -1, 0, timeout); 
	}
	
	public static short waitForFile(File file, int minSize, int timeout) { // check timeout + minimum size
		return waitForFileWithData(file, minSize, 0, timeout); 
	}
	
	public static short waitForFileWithData(File file, int minSize, int delayMeasureSize, int timeout) { // check timeout + minimum size + constant size
		int count = 0;
		
		if (timeout == 0) // infinite timeout
            timeout = (Integer.MAX_VALUE/2)-1; // we are going to multiply this number by 2 later on so it would be silly to get a negative value ;)
			
		while (true) {
			if (count > 0) {
				System.out.print("."); // every 500ms print '.'
				if (count%120 == 0) // every 120*500ms = 1 minute print a <CR>
					System.out.print("\n");
			}

			CUtils.sleep(500); // 500 ms
			if (file.exists() && file.length() > minSize) { // minSize in bytes
				long sizeMeasured = file.length();
				System.out.print("\n");
				System.out.print("Length measured: " + sizeMeasured + " bytes"); // can be 0
				
				if (delayMeasureSize > 0) { // need to check the size is constant after a specific delay (in Milliseconds)
					CUtils.sleep(delayMeasureSize);
					
					long newSizeMeasured = file.length();
					System.out.print("Length measured again: " + newSizeMeasured + " bytes");
					
					if (newSizeMeasured == sizeMeasured) { // size did not change
						break; // success
					} else {
						System.out.print("Size is changing - keep on tracking...");
					}
				} else { // no need to check constant size
					break; // success
				}
			}

			if (count>timeout*2) { // timeout is in seconds
				return RESULT_FAILURE;
			}
			count++;
		} // while true
		
		return RESULT_SUCCESS;
	}
	
	
}
