/*
+----------------------------------------------------------------------+
|                    Class: CLauncherRestBackend                       |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.etasq;

import org.json.JSONException;
import org.json.JSONObject;

import com.xqual.xcommon.IConstantsResults;

public class CLauncherRestBackend implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	public CLauncherRestConnectionManager connectionManager;
    
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+
    
	public CLauncherRestBackend(CLauncherRestConnectionManager connectionManager) {
		this.connectionManager = connectionManager;
	}
	
	// +==============================================================+
	// | GET Methods                                                  |
	// +==============================================================+
	
	public String getLibraries() {
		String jsonContent = performGetQuery("libraries/", "");
		return jsonContent;
	}
	
	public String getKeywords(String libraryId) {
		String jsonContent = performGetQuery("libraries/" + libraryId + "/keywords/", "");
		return jsonContent;
	}
	
	public String getKeyword(String libraryId, String keywordId) {
		String jsonContent = performGetQuery("libraries/" + libraryId + "/keywords/" + keywordId, "");
		return jsonContent;
	}

	public String getRun(String runId) {
		String jsonContent = performGetQuery("runs/" + runId, "");
		return jsonContent;
	}

	// +==============================================================+
	// | POST Methods                                                 |
	// +==============================================================+

	public String executeKeyword(String data) {
		String jsonContent = performPostQuery("runs/", "", data);
		return jsonContent;
	}


	
	// -------------------------------------------
	// GET AND POST TEMPLATES
	// -------------------------------------------

   	private String performGetQuery(String methodName, String parameters) {
   		System.out.println(">>>>>> [REST/GET ] " + methodName + "()");
    	try {
	    	String jsonContent = connectionManager.getData("/" + methodName + (parameters != null && parameters.trim().length()>0 ? "?" + parameters : ""));
	    		
	    	if (jsonContent == null) { // return code != 200
	    		System.err.println(">>>>>> [REST/GET ] " + methodName + "() Failure!");
	    		return null;
	    	} else {
	    		System.out.println(">>>>>> [REST/GET ] " + methodName + "() success: " /*+ jsonContent*/);
	    	}
	    	
	    	return jsonContent;
    	} catch (Exception e) {
    		System.err.println(">>>>>> [REST/GET ] " + methodName + "() Exception: " + e);
			return null;
		}
   	}
   	
   	private String performPostQuery(String methodName, String parameters, String data) {
   		System.out.println(">>>>>> [REST/POST] " + methodName + "()");
    	try {
	    	String jsonContent = connectionManager.postData("/" + methodName + (parameters != null && parameters.trim().length()>0 ? "?" + parameters : ""), data);
	    	
	    	if (jsonContent == null) { // return code != 200
	    		System.err.println(">>>>>> [REST/POST] " + methodName + "() Failure!");
	    		return null;
	    	} else {
	    		System.out.println(">>>>>> [REST/POST] " + methodName + "() success: " /*+ jsonContent*/);
	    	}
	    	
	    	return jsonContent;
    	} catch (Exception e) {
			System.err.println(">>>>>> [REST/POST] " + methodName + "() Exception: " + e);
			return null;
		}
   	}

   	private static short getShortFromJson(String jsonContent, String parameterName) {
   		if (jsonContent == null) {
    		System.out.println(">>>>>> null json! return code different than 200 or 201 received!");
   			return -1;
   		}
   		
   		System.out.println(">>>>>> json content: " + jsonContent);
   		
    	JSONObject jsonObject;
		try {
			jsonObject = new JSONObject(jsonContent);
		} catch (JSONException e) {
    		System.out.println(">>>>>> invalid json!");
   			return -1;
		}
		
    	String result = jsonObject.getString("result");
    	if (result == null) {
    		System.out.println(">>>>>> no result in json!");
    		return -1;
    	}
    	
		System.out.println(">>>>>> " + result);
    	if (result.equalsIgnoreCase("success")) {
    		try {
				return (short)jsonObject.getInt(parameterName);
			} catch (JSONException e) {
				return -1;
			}
    	} else { // should never happen as when result is not success getData() returns a null structure!
    		return -1;
    	}
   	}
   	private static int getIntegerFromJson(String jsonContent, String parameterName) {
   		if (jsonContent == null) {
    		System.out.println(">>>>>> null json! return code different than 200 or 201 received!");
   			return -1;
   		}
   		
   		System.out.println(">>>>>> json content: " + jsonContent);
   		
    	JSONObject jsonObject;
		try {
			jsonObject = new JSONObject(jsonContent);
		} catch (JSONException e) {
    		System.out.println(">>>>>> invalid json!");
   			return -1;
		}
		
    	String result = jsonObject.getString("result");
    	if (result == null) {
    		System.out.println(">>>>>> no result in json!");
    		return -1;
    	}
    	
		System.out.println(">>>>>> " + result);
    	if (result.equalsIgnoreCase("success")) {
    		try {
				return jsonObject.getInt(parameterName);
			} catch (JSONException e) {
				return -1;
			}
    	} else { // should never happen as when result is not success getData() returns a null structure!
    		return -1;
    	}
   	}
   	private static boolean getBooleanFromJson(String jsonContent, String parameterName) {
   		if (jsonContent == null) {
    		System.out.println(">>>>>> null json! return code different than 200 or 201 received!");
   			return false;
   		}
   		
   		System.out.println(">>>>>> json content: " + jsonContent);
   		
    	JSONObject jsonObject;
		try {
			jsonObject = new JSONObject(jsonContent);
		} catch (JSONException e) {
    		System.out.println(">>>>>> invalid json!");
   			return false;
		}
		
    	String result = jsonObject.getString("result");
    	if (result == null) {
    		System.out.println(">>>>>> no result in json!");
    		return false;
    	}
    	
		System.out.println(">>>>>> " + result);
    	if (result.equalsIgnoreCase("success")) {
    		try {
				return jsonObject.getBoolean(parameterName);
			} catch (JSONException e) {
				return false;
			}
    	} else { // should never happen as when result is not success getData() returns a null structure!
    		return false;
    	}
   	}
   	private static String getStringFromJson(String jsonContent, String parameterName) {
   		if (jsonContent == null) {
    		System.out.println(">>>>>> null json! return code different than 200 or 201 received!");
   			return null;
   		}
   		
   		System.out.println(">>>>>> json content: " + jsonContent);
   		
    	JSONObject jsonObject;
		try {
			jsonObject = new JSONObject(jsonContent);
		} catch (JSONException e) {
    		System.out.println(">>>>>> invalid json!");
   			return null;
		}
		
    	String result = jsonObject.getString("result");
    	if (result == null) {
    		System.out.println(">>>>>> no result in json!");
    		return null;
    	}
    	
		System.out.println(">>>>>> " + result);
    	if (result.equalsIgnoreCase("success")) {
    		try {
				return jsonObject.getString(parameterName);
			} catch (JSONException e) {
				return null;
			}
    	} else { // should never happen as when result is not success getData() returns a null structure!
    		return null;
    	}
   	}
   	private static short getResultFromJson(String jsonContent) {
   		if (jsonContent == null) {
    		System.out.println(">>>>>> null json! return code different than 200 or 201 received!");
   			return RESULT_FAILURE;
   		}
   		
   		System.out.println(">>>>>> json content: " + jsonContent);
   		
    	JSONObject jsonObject;
		try {
			jsonObject = new JSONObject(jsonContent);
		} catch (JSONException e) {
    		System.out.println(">>>>>> invalid json!");
   			return RESULT_FAILURE;
		}
		
    	String result = jsonObject.getString("result");
    	if (result == null) {
    		System.out.println(">>>>>> no result in json!");
    		return RESULT_FAILURE;
    	}
    	
		System.out.println(">>>>>> " + result);
    	if (result.equalsIgnoreCase("success")) {
    		return RESULT_SUCCESS;
    	} else { // should never happen as when result is not success getData() returns a null structure!
    		return RESULT_FAILURE;
    	}
   	}

   	private static int getIdFromJson(String jsonContent) {
   		return getIntegerFromJson(jsonContent, "id");
   	}
   	private static String getNameFromJson(String jsonContent) {
   		return getStringFromJson(jsonContent, "name");
   	}
   	private static String getMessageFromJson(String jsonContent) { // isn't there a result in the json ? if so we could reuse method above : getStringFromJson("message")
   		if (jsonContent == null) {
    		System.out.println(">>>>>> null json! return code different than 200 or 201 received!");
   			return null;
   		}

   		System.out.println(">>>>>> json content: " + jsonContent);

   		JSONObject jsonObject;
		try {
			jsonObject = new JSONObject(jsonContent);
		} catch (JSONException e) {
    		System.out.println(">>>>>> invalid json!");
   			return null;
		}
		
    	String message = jsonObject.getString("message");
    	if (message == null) {
    		System.out.println(">>>>>> no message!");
    		return null;
    	}
    	
		return message;
   	}
}

