/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Jean Marin (XQual)                                      |
| Version:     1.4                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.gatling;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;
import java.util.Hashtable;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;
import com.xqual.xlauncher.CTimeoutListener;
import com.xqual.xcommon.CZipWriter;

/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Batch files.
 * @author jmarin
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{gatling      }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;


	private String gatlingInstallPath;
	private String OSPlatform;
	private File workingDir;
	private File logFile;
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath = getStringParamValue("Gatling",  "Test root path");
			gatlingInstallPath = getStringParamValue("Gatling",  "Gatling install path");
			OSPlatform = getStringParamValue("Gatling",  "OS");
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		String scriptParentFolderPath = testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/";
		workingDir = new File(scriptParentFolderPath);
		
		String 	gatlingInterpreter = "";
		gatlingInterpreter = gatlingInterpreter + CFileUtils.quoteFilePath((gatlingInstallPath.length() > 0 ? gatlingInstallPath + "/bin/gatling" + (OSPlatform.equals("Windows") ? ".bat" : ".sh") : "")) + " " ;
		gatlingInterpreter = gatlingInterpreter + "-sf ";
		gatlingInterpreter = gatlingInterpreter + CFileUtils.quoteFilePath(testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/") + " ";
		gatlingInterpreter = gatlingInterpreter + "-s ";
		gatlingInterpreter = gatlingInterpreter + testName + " ";
		gatlingInterpreter = gatlingInterpreter + "-m ";
		gatlingInterpreter = gatlingInterpreter + "-rd ";
		gatlingInterpreter = gatlingInterpreter + CFileUtils.quoteFilePath("run " + testName) + " "; // ajouter le  time ?
		gatlingInterpreter = gatlingInterpreter + "-on ";
		gatlingInterpreter = gatlingInterpreter + CFileUtils.quoteFilePath(workingDir.getAbsolutePath() + "/" + "xstudiogatling");
					
		
		// C:\gatling\machin\Mes simulation 1\simulations\computerdatabase>"C:\gatling\g2.3.1\gatling\bin\gatling.bat" -sf "c:\gatling\machin\mes simulations 1\simulations\computerdatabase\advanced"  -s computerdatabase.advanced.AdvancedSimulationStep01 -m -rd "caroule" -on "c:\gatling\machin\pitchou"
		
		//todo: set the envrionment for GATLING_HOME
		// for this we need to pass teh envrionement to the runner
		Hashtable<String, String> env = new Hashtable<String, String>();
		env.put("GATLING_HOME", gatlingInstallPath);
		env.put("NO_PAUSE", "no pause");
		
		
		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		CRunner gatlingRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
                                        gatlingInterpreter,
										env,
                                        workingDir);

		logFile = new File (workingDir + "/XstudioLogFile.txt");
		
		short result = RESULT_FAILURE;
		
		result = gatlingRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, logFile);
		traceln(LOG_PRIORITY_INFO, "returned from gatling  runner!");
		
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution failed"));
			parseResultFile(executionSteps);
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution succeeded"));
			parseResultFile(executionSteps);
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}	
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		Vector<CScannedTest> gatlingOutput = new Vector<CScannedTest>();
		File gatlingScalaFile;
		String gatlingCanonicalPath;
		String gatlingRelativePath;
		String gatlingSimulationName;		
		String gatlingFileAsString;
		String scalaClassFullyQualifiedName;
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2escala$",          // only *.scala files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		
		traceln(LOG_PRIORITY_INFO, "found scala files - trying to convert test names !");
		
		for (CScannedTest scannedTest : output) {
			gatlingCanonicalPath = scannedTest.getCanonicalPath();
			gatlingRelativePath = scannedTest.getRelativePath();
			gatlingSimulationName = scannedTest.getName();
			
			traceln(LOG_PRIORITY_INFO, "searching in scala file !");
			
			gatlingScalaFile = new File(testRootPathFolder + "/" + (gatlingCanonicalPath.length() > 0 ? gatlingCanonicalPath  + "/" : "" ) + gatlingSimulationName + ".scala");
			
			scalaClassFullyQualifiedName = extractPackageAndClassFromScalaFile (gatlingScalaFile);
			traceln(LOG_PRIORITY_INFO, "got the string to create a scanned test " + scalaClassFullyQualifiedName + " : " + gatlingCanonicalPath);
			gatlingOutput.add( new CScannedTest(scalaClassFullyQualifiedName, gatlingCanonicalPath));
		
			
		}
		
		
		return gatlingOutput;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		// parse the result file to get the result and the execution steps
		if (!logFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "log file exists"));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
			addTestcaseMessageAttachment(logFile);
		}

		String line, message="";
		boolean errorDetected = false;

		FileInputStream fileInputStream = null;
		BufferedReader bufferedReader = null;
		String reportIndexHTMLFileName = "";
		try {
			fileInputStream = new FileInputStream(logFile);
			bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);
				if (line.indexOf("request count ")>=0 || 
				line.indexOf("min response time")>=0 ||
				line.indexOf("max response time")>=0 ||
				line.indexOf("mean response time")>=0 ||
				line.indexOf("std deviation")>=0 ||
				line.indexOf("mean requests/sec")>=0 ||
				line.indexOf("failed ")>=0 ) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));

				} else if (line.indexOf("Simulation")>=0 && line.indexOf("completed in")>=0) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));
				} else if (line.indexOf("Please open the following file: ")>=0) {
					reportIndexHTMLFileName = line.substring(32, line.length()).trim();
				} else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}
			
			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception whle parsing the result file: " + e));
			errorDetected = true;
		}
		
		CZipWriter zippedHTML = new CZipWriter(workingDir.getAbsolutePath() + "/index.html.zip");
		zippedHTML.zipFolder(new File (reportIndexHTMLFileName).getParent());
		addTestcaseMessageAttachment(new File(workingDir.getAbsolutePath() + "/index.html.zip"));
		

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
	
	
	private String extractPackageAndClassFromScalaFile(File scalaFile) {
		// get a file , opens it , find the package , find the first class and concatenate this as a package.clasname
		String fileAsString;
		traceln(LOG_PRIORITY_INFO, "scala file converted to string !" + scalaFile.getAbsolutePath() );
		fileAsString = CFileUtils.FileToString(scalaFile);
		traceln(LOG_PRIORITY_INFO, "scala file converted to string -----" + fileAsString);
		
		String packageName = "";
		String className = "";
		String line = "";

		FileInputStream fileInputStream = null;
		BufferedReader bufferedReader = null;
		try {
			fileInputStream = new FileInputStream(scalaFile);
			bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				if (line.indexOf("package ")>=0) {
					try {packageName = line.substring(8, line.length());} catch (Exception e) {} // [Success] length = 9
				} else if (line.indexOf("class ")>=0) {
					try {className = line.substring(6, line.indexOf("extends Simulation"));} catch (Exception e) {}
				} else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}
			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the scala file: " + e);
		}
		
		String scalaClassFullyQualifiedName;
		packageName = packageName.trim();
		className = className.trim();
		scalaClassFullyQualifiedName = packageName + "." + className;
		traceln(LOG_PRIORITY_INFO, "got scalaClassFullyQualifiedName  as " + scalaClassFullyQualifiedName);
		
		return scalaClassFullyQualifiedName;	
	}
	
}
