/*
+----------------------------------------------------------------------+
|                    Class: CLauncherRestBackend                       |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.vtaas;

import com.xqual.xcommon.IConstantsResults;

public class CLauncherRestBackend implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	public CLauncherRestConnectionManager connectionManager;
    
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+
    
	public CLauncherRestBackend(CLauncherRestConnectionManager connectionManager) {
		this.connectionManager = connectionManager;
	}
	
	// +==============================================================+
	// | GET Methods                                                  |
	// +==============================================================+
	
	public String getStatus(int testcaseRunId) {
		String jsonContent = performGetQuery("status", "/"+testcaseRunId);
		return jsonContent;
	}

	
	// +==============================================================+
	// | POST Methods                                                 |
	// +==============================================================+

	public String run(String jsonInput) {
		String jsonContent = performPostQuery("run", "", jsonInput);
		return jsonContent;
	}

	
	// -------------------------------------------
	// GET AND POST TEMPLATES
	// -------------------------------------------

   	private String performGetQuery(String methodName, String parameters) {
   		System.out.println(">>>>>> [REST/GET ] " + methodName + "()");
    	try {
	    	String jsonContent = connectionManager.getData("/" + methodName + parameters);
	    	
	    	if (jsonContent == null) { // return code != 200
	    		System.err.println(">>>>>> [REST/GET ] " + methodName + "() Failure!");
	    		return null;
	    	} else {
	    		System.out.println(">>>>>> [REST/GET ] " + methodName + "() success: " + jsonContent);
	    	}
	    	
	    	return jsonContent;
    	} catch (Exception e) {
    		System.err.println(">>>>>> [REST/GET ] " + methodName + "() Exception: " + e);
			return null;
		}
   	}
   	
   	private String performPostQuery(String methodName, String parameters, String data) {
   		System.out.println(">>>>>> [REST/POST] " + methodName + "()");
    	try {
	    	String jsonContent = connectionManager.postData("/" + methodName + parameters, data);
	    	
	    	if (jsonContent == null) { // return code != 200
	    		System.err.println(">>>>>> [REST/POST] " + methodName + "() Failure!");
	    		return null;
	    	} else {
	    		System.out.println(">>>>>> [REST/POST] " + methodName + "() success: " + jsonContent);
	    	}
	    	
	    	return jsonContent;
    	} catch (Exception e) {
			System.err.println(">>>>>> [REST/POST] " + methodName + "() Exception: " + e);
			return null;
		}
   	}
}

