/*
+----------------------------------------------------------------------+
|                    Class: CLauncherRestConnectionManager             |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.vtaas;

import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;

import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.network.CYesHostnameVerifier;
import com.xqual.xcommon.utils.network.CYesTrustManager;

public class CLauncherRestConnectionManager {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+
	
	private final String baseUrl;
	private final String apiKey;

	private final boolean httpsEnabled;
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+
	
	public CLauncherRestConnectionManager(String baseUrl, String apiKey) {
	    this.baseUrl = baseUrl; // i.e. "https://vtaas-gateway.osc-fr1.scalingo.io"
		this.apiKey  = apiKey;  // i.e. "fa7aa208-0dd2-4cc5-9839-b32868ab1d04"

	    HostnameVerifier yesHostnameVerifier = new CYesHostnameVerifier();
	    TrustManager yesTrustManager = new CYesTrustManager();
        
        URL url = null;
	    try {
	    	url = new URL(baseUrl);
		} catch (Throwable t) {
			t.printStackTrace();
			httpsEnabled= false;
			return;
		}
	    
	    // ------------------------------
	    // configure the HTTPS connection
	    // ------------------------------
	    if (url.getProtocol().equalsIgnoreCase("https")) {
			System.out.println("the VTaaS server is an https/SSL server");
		    httpsEnabled = true;
		    
		    // USE THE SAME SETTINGS THAN FOR DATABASE (database.connection.accept_any_certificate)
		    //if (CStudioGlobals.applicationConfProperties.isAcceptAnyCertificate()) { // the user accepts ANY certificate
		    	System.out.println("==> accepting ANY certificate");
		        // initiate the SSL context and prepare trust manager that will accept ANY certificate
		        TrustManager[] yesTrustManagers = new TrustManager[] {yesTrustManager};
		        try {
					SSLContext yesSslcontext = SSLContext.getInstance("TLS");
					yesSslcontext.init(null, yesTrustManagers, null); // key manager, trust manager, secure random manager
					SSLSocketFactory yesSslFactory = yesSslcontext.getSocketFactory();
					
				    HttpsURLConnection.setDefaultSSLSocketFactory(yesSslFactory);
				    HttpsURLConnection.setDefaultHostnameVerifier(yesHostnameVerifier);
		        } catch (Exception e) {
				}
		        
		    /*} else {
		    	System.out.println(" ==> accepting only certificate matching a certificate from the local/private keystore");
		    	HttpsURLConnection.setDefaultSSLSocketFactory(CStudioGlobals.xstudioSslFactory); // this will trust the server only if its certificates has been added in the conf
		    }*/

	    // -----------------------------
	    // configure the HTTP connection
	    // -----------------------------
	    } else {
	    	System.out.println("the XStudio server is an http server");
		    httpsEnabled = false;
	    }
	}
	
	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	public String getData(String urlPostfix) {
		HttpURLConnection connectionGet = null;
		try {
	        // prepare the connection
	        URL url = new URL(baseUrl + urlPostfix);
	        System.out.println("connecting to url \"" + url + "\"");
	        
	        // the connection may already be opened so we catch the IllegalStateException "Already Connected"
	        try {
				if (httpsEnabled) {
					connectionGet = (HttpsURLConnection)url.openConnection(); // HttpsURLConnection extends HttpURLConnection

				} else if (url.getProtocol().equalsIgnoreCase("http")) {
					connectionGet = (HttpURLConnection)url.openConnection();
				}
	        
	            connectionGet.setRequestMethod("GET");
	            /*
	            http://stackoverflow.com/questions/17278303/basic-authentication-fails-with-glassfish
	            Oracle has enabled caching of HTTP responses by default with JDK7 in Web Start: 
				Caching enabled by default: Caching of network content for application code running in Web Start mode is now enabled by default. 
				This allows application improved performance and consistency with applet execution mode. 
				To ensure the latest copy of content is used, the application can use URLConnection.setUseCaches(false) or request header Cache-Control values no-cache/no-store.
				*/
	            // maybe we should try to re-enable cache after the first connection?
		        connectionGet.setRequestProperty("Cache-Control", "no-cache");
		        connectionGet.setRequestProperty("Cache-Control", "no-store");
		        connectionGet.setRequestProperty("X-API-KEY", apiKey);
		        connectionGet.setRequestProperty("Connection", "keep-alive");
		        
	        } catch (IllegalStateException ise) {
				System.out.println("Already connected\n");
			}
	        
	        // ----------- CONNECTION -----------
	        connectionGet.connect(); // that's where security handshake is done

	        // store the code and the message so that the caller can retrieve this
	        int responseCode = connectionGet.getResponseCode();
	        String responseMessage = connectionGet.getResponseMessage();

	        if (responseCode != 200) {
	        	System.err.println("Server response: " + responseCode + " (" + responseMessage + ")");
	        } else {
	        	//System.out.println("Server response: " + responseCode + " (" + responseMessage + ")");
	        }
	        
	        if (responseCode != 200) {
	        	String output = CFileUtils.streamToString(connectionGet.getErrorStream());
		        System.err.println("ERROR=" + output);
				try {
					System.err.println("Disconnecting...");
					connectionGet.disconnect();
				} catch (Exception e) {
					System.err.println("Couldn't disconnect...");
				}
				System.err.println("returning null...");
	        	return null;
	        }
	        
	        String output = CFileUtils.streamToString(connectionGet.getInputStream());
	        //System.err.println("OUTPUT=" + output);
	        connectionGet.disconnect();
	        return output;
	        
		} catch (Throwable t) {
			System.err.println("Exception while communicating with VTaaS REST server: " + t);
			try {
				System.err.println("Disconnecting...");
				connectionGet.disconnect();
			} catch (Exception e) {
				System.err.println("Couldn't disconnect...");
			}
			return null;
		}
	}	

	/*
	public String downloadData(String urlPostfix, File outputFile) {
		HttpURLConnection connectionGet = null;
		try {
	        // prepare the connection
	        URL url = new URL(baseUrl + urlPostfix);
	        System.out.println("connecting to url \"" + url + "\"");
	        
	        // the connection may already be opened so we catch the IllegalStateException "Already Connected"
	        try {
				if (httpsEnabled) {
					connectionGet = (HttpsURLConnection)url.openConnection(); // HttpsURLConnection extends HttpURLConnection

				} else if (url.getProtocol().equalsIgnoreCase("http")) {
					connectionGet = (HttpURLConnection)url.openConnection();
				}
	        
	            connectionGet.setRequestMethod("GET");
	            // maybe we should try to re-enable cache after the first connection?
		        connectionGet.setRequestProperty("Cache-Control", "no-cache");
		        connectionGet.setRequestProperty("Cache-Control", "no-store");
		        connectionGet.setRequestProperty("apikey", apiKey);
		        connectionGet.setRequestProperty("Connection", "keep-alive");
		        
	        } catch (IllegalStateException ise) {
				System.out.println("Already connected\n");
			}
	        
	        addCookie(connectionGet);
	        
	        // ----------- CONNECTION -----------
	        connectionGet.connect(); // that's where security handshake is done
	        //if (httpsEnabled) {
	        //    CStudioSqUtils.printHttpsCertificateChain((HttpsURLConnection)connection); //dump all cert info
	        //}

	        // store the code and the message so that the caller can retrieve this
	        int responseCode = connectionGet.getResponseCode();
	        String responseMessage = connectionGet.getResponseMessage();

	        if (responseCode != 200) {
	        	System.err.println("Server response: " + responseCode + " (" + responseMessage + ")");
	        } else {
	        	//System.out.println("Server response: " + responseCode + " (" + responseMessage + ")");
	        }
	        
	        if (responseCode != 200) {
	        	String output = CFileUtils.streamToString(connectionGet.getErrorStream());
		        System.err.println("ERROR=" + output);
				try {
					System.err.println("Disconnecting...");
					connectionGet.disconnect();
				} catch (Exception e) {
					System.err.println("Couldn't disconnect...");
				}
	        	return null;
	        }
	        
	        CFileUtils.streamToFile(connectionGet.getInputStream(), outputFile);
	        //System.err.println("OUTPUT=" + output);
	        connectionGet.disconnect();
	        return null;
	        
		} catch (Throwable t) {
			System.err.println("Exception while communicating with XStudio REST server: " + t);
			try {
				System.err.println("Disconnecting...");
				connectionGet.disconnect();
			} catch (Exception e) {
				System.err.println("Couldn't disconnect...");
			}
			return null;
		}
	}	
*/
	
	public String postData(String urlPostfix, String jsonData) {
		HttpURLConnection connectionPost = null;
		try {
	        // prepare the connection
	        URL url = new URL(baseUrl + urlPostfix);
	        System.out.println("connecting to url \"" + url + "\"");
	        
	        // the connection may already be opened so we catch the IllegalStateException "Already Connected"
	        try {
				if (httpsEnabled) {
					connectionPost = (HttpsURLConnection)url.openConnection(); // HttpsURLConnection extends HttpURLConnection

				} else if (url.getProtocol().equalsIgnoreCase("http")) {
					connectionPost = (HttpURLConnection)url.openConnection();
				}
	        
	            connectionPost.setRequestMethod("POST");
	            /*
	            http://stackoverflow.com/questions/17278303/basic-authentication-fails-with-glassfish
	            Oracle has enabled caching of HTTP responses by default with JDK7 in Web Start: 
				Caching enabled by default: Caching of network content for application code running in Web Start mode is now enabled by default. 
				This allows application improved performance and consistency with applet execution mode. 
				To ensure the latest copy of content is used, the application can use URLConnection.setUseCaches(false) or request header Cache-Control values no-cache/no-store.
				*/
	            // maybe we should try to re-enable cache after the first connection?
		        connectionPost.setRequestProperty("Cache-Control", "no-cache");
		        connectionPost.setRequestProperty("Cache-Control", "no-store");
		        connectionPost.setRequestProperty("X-API-KEY", apiKey);
		        connectionPost.setRequestProperty("Connection", "keep-alive");
		        
		        // format of the body
		        connectionPost.setRequestProperty("Content-Type", "application/json;charset=utf-8");

				// Send post request
		        connectionPost.setDoInput(true);
		        connectionPost.setDoOutput(true);
				BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(connectionPost.getOutputStream(), "UTF-8"));
				
				bw.write(jsonData);
				//bw.write(TOTO);
				bw.flush();
				bw.close();
	        } catch (IllegalStateException ise) {
				System.out.println("Already connected\n");
			}
	        
	        
	        // ----------- CONNECTION -----------
	        connectionPost.connect(); // that's where security handshake is done

	        // store the code and the message so that the caller can retrieve this
	        int responseCode = connectionPost.getResponseCode();
	        String responseMessage = connectionPost.getResponseMessage();

	        if (responseCode != 200 && responseCode != 201) { // 201=Created
	        	System.err.println("Server response: " + responseCode + " (" + responseMessage + ")");
	        } else {
	        	//System.out.println("Server response: " + responseCode + " (" + responseMessage + ")");
	        }
	        
	        if (responseCode != 200 && responseCode != 201) { // 201=Created
	        	//String output = CFileUtils.streamToString(connectionPost.getErrorStream());
		        //System.err.println("ERROR=" + output);
				try {
					System.err.println("Disconnecting...");
					connectionPost.disconnect();
				} catch (Exception e) {
					System.err.println("Couldn't disconnect...");
				}
				System.err.println("returning null...");
	        	return null;
	        }
	        
	        String output = CFileUtils.streamToString(connectionPost.getInputStream());
	        //System.err.println("OUTPUT=" + output);
	        connectionPost.disconnect();
	        return output;
	        
		} catch (Throwable t) {
			System.err.println("Exception while communicating with XStudio REST server: " + t);
			try {
				System.err.println("Disconnecting...");
				connectionPost.disconnect();
			} catch (Exception e) {
				System.err.println("Couldn't disconnect...");
			}
			return null;
		}
	}

/*
	public String postDataAttach(String urlPostfix, File file) {
		String crlf 		= "\r\n";
		String twoHyphens 	= "--";
		String boundary 	=  "*****";
		
		HttpURLConnection connectionPost = null;
		try {
	        // prepare the connection
	        URL url = new URL(baseUrl + urlPostfix);
	        System.out.println("connecting to url \"" + url + "\"");
	        
	        // the connection may already be opened so we catch the IllegalStateException "Already Connected"
	        try {
				if (httpsEnabled) {
					connectionPost = (HttpsURLConnection)url.openConnection(); // HttpsURLConnection extends HttpURLConnection

				} else if (url.getProtocol().equalsIgnoreCase("http")) {
					connectionPost = (HttpURLConnection)url.openConnection();
				}
	        
	            connectionPost.setRequestMethod("POST");
	            // maybe we should try to re-enable cache after the first connection?
		        connectionPost.setRequestProperty("Cache-Control", "no-cache");
		        connectionPost.setRequestProperty("Cache-Control", "no-store");
		        connectionPost.setRequestProperty("Authorization", encodedAuth);
		        connectionPost.setRequestProperty("Connection", "keep-alive");
		        
		        connectionPost.setUseCaches(false);
		        connectionPost.setDoInput(true);
		        
		        // format of the body
		        //connectionPost.setRequestProperty("content-disposition",		"form-data; name=\"" + "file" + "\"; filename=\"" + file.getName() + "\"");
		        connectionPost.setRequestProperty("Content-Type", 				"multipart/form-data;boundary=" + boundary);
		        connectionPost.setRequestProperty("Content-Transfer-Encoding",	"binary");
		        
		        addCookie(connectionPost);

				// Send post request
		        connectionPost.setDoOutput(true);
		        
		        DataOutputStream dataOutputStream = new DataOutputStream(connectionPost.getOutputStream());
	        	dataOutputStream.writeBytes(twoHyphens + boundary + crlf);
	        	dataOutputStream.writeBytes("Content-Disposition: form-data; name=\"" + "file" + "\";filename=\"" + file.getName() + "\"" + crlf);
	        	dataOutputStream.writeBytes(crlf);
	        	dataOutputStream.write(Files.readAllBytes(file.toPath()));
	        	dataOutputStream.writeBytes(crlf);
	        	dataOutputStream.writeBytes(twoHyphens + boundary + twoHyphens + crlf);
	        	dataOutputStream.flush();
	        	dataOutputStream.close();
		        
	        } catch (IllegalStateException ise) {
				System.out.println("Already connected\n");
			}
	        
	        
	        // ----------- CONNECTION -----------
	        connectionPost.connect(); // that's where security handshake is done
	        //if (httpsEnabled) {
	        //    CStudioSqUtils.printHttpsCertificateChain((HttpsURLConnection)connectionPost); //dump all cert info
	        //}

	        // store the code and the message so that the caller can retrieve this
	        int responseCode = connectionPost.getResponseCode();
	        String responseMessage = connectionPost.getResponseMessage();

	        if (responseCode != 200 && responseCode != 201) { // 201=Created
	        	System.err.println("Server response: " + responseCode + " (" + responseMessage + ")");
	        } else {
	        	//System.out.println("Server response: " + responseCode + " (" + responseMessage + ")");
	        }
	        
	        if (responseCode != 200 && responseCode != 201) { // 201=Created
	        	//String output = CFileUtils.streamToString(connectionPost.getErrorStream());
		        //System.err.println("ERROR=" + output);
				try {
					System.err.println("Disconnecting...");
					connectionPost.disconnect();
				} catch (Exception e) {
					System.err.println("Couldn't disconnect...");
				}
	        	return null;
	        }
	        
	        String output = CFileUtils.streamToString(connectionPost.getInputStream());
	        //System.err.println("OUTPUT=" + output);
	        connectionPost.disconnect();
	        return output;
	        
		} catch (Throwable t) {
			System.err.println("Exception while communicating with XStudio REST server: " + t);
			try {
				System.err.println("Disconnecting...");
				connectionPost.disconnect();
			} catch (Exception e) {
				System.err.println("Couldn't disconnect...");
			}
			return null;
		}
	}
*/

	/*
	private void addCookie(HttpURLConnection connection) {
        boolean cookieAlreadyStoredLocally = false;
        
        // Cookie: local store --> message
        String storedCookie = null;
        try {
    		System.out.println("local cookie store:" + cookieStore);
    		if (cookieStore != null) {
    			List<HttpCookie> cookiesList = cookieStore.getCookies();
        		System.out.println("local cookies list:" + cookiesList);
        		if (cookiesList != null) {
        			System.out.println("cookies# in the local store:" + cookiesList.size());
        			if (cookiesList.size() > 0) {
	        			HttpCookie firstCookie = cookiesList.get(0);
	        			if (firstCookie != null) {
	        				storedCookie = firstCookie.toString();
	        				System.out.println(">>> set cookie in the message: " + storedCookie);
	        				connection.setRequestProperty("Cookie", storedCookie);
	        				cookieAlreadyStoredLocally = true;
	        			} else {
		        			System.err.println("null cookie in local store!");
		        		}
        			} else {
	        			System.out.println("no cookie available in local store"); // at the first connection, this is normal
	        		}
        		} else {
        			System.err.println("null local cookies list!");
        		}
    		} else {
    			System.err.println("null local cookie store!");
    		}
        } catch (Exception e) {
        	System.err.println("Exception while parsing cookies: " + e);
        }

        if (!cookieAlreadyStoredLocally) {
	        // Cookie: message --> local store
	        Map<String, List<String>> headerFields = connection.getHeaderFields();
	        System.out.println("header fields: " + headerFields);
	        
	        List<String> cookiesHeader = headerFields.get("Set-Cookie");
	        System.out.println("cookies header: " + cookiesHeader);
	        //System.out.println("----------------------------------------------------------------------");
	        
	        if (cookieStore != null) {
	        	if (cookiesHeader != null) {
			        if (cookiesHeader.size() > 0) {
			        	for (String cookieHeader : cookiesHeader) {
			        		System.out.println("cookie header: " + cookieHeader);
			        		if (cookieHeader != null && cookieHeader.length() > 0) {
			        			List<HttpCookie> cookiesList = HttpCookie.parse(cookieHeader);
			        			System.out.println("cookies list:" + cookiesList);
				        		if (cookiesList != null) {
				        			System.out.println("cookies# in the header:" + cookiesList.size());
				        			if (cookiesList.size() > 0) {
				        				
										for (HttpCookie cookie : cookiesList) { // REST message may includes several cookies
											if (cookie != null) {
												try {
													if (cookie.toString().startsWith("JSESSIONID")) { // we need to store ONLY this one !!!
														System.err.println("<<< saving cookie in the local store: " + cookie.toString());
														cookieStore.add(null, cookie);
													}
												} catch (Exception e) {
													System.out.println("couldn't store cookie!");
												};
											}
										}

				        			} else {
						    			System.out.println("no cookie available in the message");
						    		}
				        		} else {
					    			System.err.println("null cookies list in the message!");
					    		}
			        		} else {
				    			System.err.println("null or empty cookie header in the message!");
				    		}
			        		//System.out.println("----------------------------------------------------------------------");
			        	}
			        } else {
		    			System.err.println("no cookie header in the message!");
		    		}
		        } else {
		        	System.out.println("null cookies header in the message"); // in the next connections, this is normal
	    		}
	        } else {
    			System.err.println("null local cookie store!");
    		}
        }
	}
	*/
	
	public String getBaseUrl() {
		return baseUrl;
	}

}
